==> dmail-1.14.tgz: dmail/do_lists.c <==

/*
 *  DO_LISTS.C
 *
 *  Global Routines:    DO_SETLIST()
 *                      DO_LIST()
 *                      DO_SELECT()
 *                      DO_DEFER()
 *
 *  Static Routines:    None.
 *
 *      LIST associated commands.
 *
 */

#include <stdio.h>
#include <string.h>
#include "dmail.h"

Prototype int do_setlist(char *str, int com);
Prototype int do_rlist(char *garbage, int com);
Prototype int do_list(char *garbage, int com);
Prototype int do_select(char *str, int mode);
Prototype int do_defer(char *garbage, int com);

char *findSig(const char *str, int l);

int
do_setlist(char *str, int com)
{
    int i, fw, idx, localecho = 1;
    int sac = 1;

    push_break();
    if (ac > sac && strcmp (av[sac], "-s") == 0) {
        ++sac;
        localecho = 0;
    }
    hold_load();
    if (ac > sac) {
        Listsize = 0;
        for (i = sac; i < ac; ++i) {
            fw = atoi(av[i]);
            if (fw > 4096)
                fw = 4096;
            if (fw < 0  ||  (*av[i] < '0'  ||  *av[i] > '9'))
                fw = 20;
            else
                ++i;
            if (i >= ac)
                continue;
            idx = get_extra_ovr (av[i]);
            if (idx < 0) {
		if (idx == -2)
		    printf ("cannot setlist article body: %s\n", av[i]);
		else
		    printf ("To many entries, cannot load: %s\n", av[i]);
                fflush (stdout);
                continue;
            }
            header[Listsize] = idx;
            width[Listsize] = fw;
            ++Listsize;
        }
    }
    nohold_load();
    pop_break();
    if (localecho) {
        puts ("");
        printf ("Entry   Width   Field\n\n");
        for (i = 0; i < Listsize; ++i) 
            printf ("%-6d   %-5d   %s\n", i, width[i], Find[header[i]].search);
        puts ("");
    }
    return (1);
}


/*
 * Pre-position #   0 >     Current article
 *                  1 -     Read already
 */

int
do_rlist(char *garbage, int com)
{
    int num = 20;
    int istart = Current, iend = Current;
    short dir = 1;
    short try;
    char buf[64];

    if (av[1])
	num = atoi(av[1]);
    if (num < 0) {
	dir = -1;
	num = -num;
    }
    for (try = 0; try < 2 && num; ++try, (dir = -dir)) {
	int i;
	if (dir < 0) {
	    for (i = Current; i >= 0 && num; --i) {
		if (Entry[i].no && !(Entry[i].status & ST_DELETED)) {
		    istart = i;
		    --num;
		}
	    }
	} else {
	    for (i = Current; i < Entries && num; ++i) {
		if (Entry[i].no && !(Entry[i].status & ST_DELETED)) {
		    iend = i;
		    --num;
		}
	    }
	}
    }
    if (istart != iend) {
	sprintf(buf, "%d-%d", Entry[istart].no, Entry[iend].no);
	ac = 2;
	av[1] = buf;
	return(do_list(NULL, 0));
    } else {
        return(-1);
    }
}

int
do_list(char *garbage, int com)
{
    int i, j;
    static char curr[10] = { "    " };

    if (ac == 1) {
        av[1] = "all";
        ++ac;
    }
    if (push_base()) {
        push_break();
        pop_base();
        PAGER ((void *)-1);
        pop_break();
        return (-1);
    }
    PAGER (0);
    FPAGER ("\n         ");
    for (j = 0; j < Listsize; ++j) {
        if (width[j]) {
            sprintf (Puf, "%-*.*s",
                    2 + width[j],
                    2 + width[j],
                    Find[header[j]].search);
            FPAGER (Puf);
        }
    }
    FPAGER ("\n");
    rewind_range (1);
    while ((i = get_range()) != 0) {
        i = indexof(i);
        if (Entry[i].no  &&  ((Entry[i].status & ST_DELETED) == 0)) {
            curr[0] = (Entry[i].status & ST_TAG) ? 'T' : ' ';
            curr[1] = (i == Current) ? '>' : ' ';
            curr[2] = (Entry[i].status & ST_READ)    ? 'r' : ' ';
            curr[3] = (Entry[i].status & ST_STORED)  ? 'w' : ' ';
            sprintf (Puf, "%s%-3d", curr, Entry[i].no);
            FPAGER (Puf);
            for (j = 0; j < Listsize; ++j) {
                if (width[j]) {
                    const char *s = Entry[i].fields[header[j]];
		    int l = data_line_length(s) - 1;

                    if (l >= 5 && strncasecmp(Find[header[j]].search, "From:", 5) == 0) {
                        s = findSig(s, l);
			l = strlen(s);
		    }
		    if (l > width[j])
			l = width[j];

		    FPAGER("  ");
		    WPAGER(s, l);
		    if (l < width[j]) {
			l = width[j] - l;
			sprintf(Puf, "%*.*s", l, l, "");
			WPAGER(Puf, l);
		    }
                }
            }
            FPAGER ("\n");
        }
    }
    FPAGER ("\n");
    PAGER ((void *)-1);
    pop_base();
    return (1);
}

int
do_select(char *str, int mode)
{
    int ret = 1;
    int localecho = 1;
    int avi = 1;
    int scr;

    if (ac == 1)
        return (1);
    if (strcmp (av[avi], "-s") == 0) {
        localecho = 0;
        ++avi;
        --ac;
    }
    switch (ac) {
    case 2:
        if (localecho)
            puts ("SELECT ALL");
        ret = m_select (Nulav, mode);
        break;
    case 1:
        break;
    default:
        ret = m_select (av + avi, mode);
        scr = indexof(0);
        if (scr > 0  &&  localecho)
            printf ("%d  Entries selected\n", Entry[scr].no);
        break;
    }
    if (ret < 0  &&  localecho) {
        puts ("Null field");
        return (-1);
    }
    return (1);
}

int
do_defer(char *garbage, int com)
{
    register int i, j;

    push_break();
    j = 1;
    for (i = 0; i < Entries; ++i) {
        if (Entry[i].no)
            Entry[i].no = (Entry[i].status & ST_READ) ? 0 : j++;
    }
    pop_break();
    return (1);
}

/*
 * findSig - find text name portion of an email address
 *
 * <email> NAME
 * NAME <email>
 * email (NAME)
 * (NAME) email
 */

char *
findSig(const char *str, int l)
{
    static char buf[256];
    char *ptr = buf;
    char *t;

    if (l > sizeof(buf) - 1)
	l = sizeof(buf) - 1;

    strncpy(buf, str, l);
    buf[l] = 0;

    while (*ptr == ' ' || *ptr == '\t')
        ++ptr;
    if (*ptr == '(') {
        if ((t = strchr(ptr, ')')) != NULL)
            *t = 0;
        return(ptr + 1);
    }
    if (*ptr == '<') {
        while (*ptr && *ptr != '>')
            ++ptr;
	if (*ptr)
	    ++ptr;
	return(ptr);
    }
    while (*ptr && *ptr != '(')
        ++ptr;
    if (*ptr == '(') {
        if ((t = strchr(ptr, ')')) != NULL)
            *t = 0;
        return(ptr + 1);
    }

    if ((t = strchr(buf, '<')) != NULL) {
        *t = 0;
    }
    return(buf);
}

==> dmail-1.23.tgz: dmail/do_lists.c <==

/*
 *  DO_LISTS.C
 *
 *  Global Routines:    DO_SETLIST()
 *                      DO_LIST()
 *                      DO_SELECT()
 *                      DO_DEFER()
 *
 *  Static Routines:    None.
 *
 *      LIST associated commands.
 *
 */

#include <stdio.h>
#include <string.h>
#include "dmail.h"

Prototype int do_setlist(char *str, int com);
Prototype int do_rlist(char *garbage, int com);
Prototype int do_list(char *garbage, int com);
Prototype int do_select(char *str, int mode);
Prototype int do_defer(char *garbage, int com);

char *findSig(const char *str, int l);

int
do_setlist(char *str, int com)
{
    int i, fw, idx, localecho = 1;
    int sac = 1;

    push_break();
    if (ac > sac && strcmp (av[sac], "-s") == 0) {
        ++sac;
        localecho = 0;
    }
    hold_load();
    if (ac > sac) {
        Listsize = 0;
        for (i = sac; i < ac; ++i) {
            fw = atoi(av[i]);
            if (fw > 4096)
                fw = 4096;
            if (fw < 0  ||  (*av[i] < '0'  ||  *av[i] > '9'))
                fw = 20;
            else
                ++i;
            if (i >= ac)
                continue;
            idx = get_extra_ovr (av[i]);
            if (idx < 0) {
		if (idx == -2)
		    printf ("cannot setlist article body: %s\n", av[i]);
		else
		    printf ("To many entries, cannot load: %s\n", av[i]);
                fflush (stdout);
                continue;
            }
            header[Listsize] = idx;
            width[Listsize] = fw;
            ++Listsize;
        }
    }
    nohold_load();
    pop_break();
    if (localecho) {
        puts ("");
        printf ("Entry   Width   Field\n\n");
        for (i = 0; i < Listsize; ++i) 
            printf ("%-6d   %-5d   %s\n", i, width[i], Find[header[i]].search);
        puts ("");
    }
    return (1);
}


/*
 * Pre-position #   0 >     Current article
 *                  1 -     Read already
 */

int
do_rlist(char *garbage, int com)
{
    int num = 20;
    int istart = Current, iend = Current;
    short dir = 1;
    short try;
    char buf[64];

    if (av[1])
	num = atoi(av[1]);
    if (num < 0) {
	dir = -1;
	num = -num;
    }
    for (try = 0; try < 2 && num; ++try, (dir = -dir)) {
	int i;
	if (dir < 0) {
	    for (i = Current; i >= 0 && num; --i) {
		if (Entry[i].no && !(Entry[i].status & ST_DELETED)) {
		    istart = i;
		    --num;
		}
	    }
	} else {
	    for (i = Current; i < Entries && num; ++i) {
		if (Entry[i].no && !(Entry[i].status & ST_DELETED)) {
		    iend = i;
		    --num;
		}
	    }
	}
    }
    if (istart != iend) {
	sprintf(buf, "%d-%d", Entry[istart].no, Entry[iend].no);
	ac = 2;
	av[1] = buf;
	return(do_list(NULL, 0));
    } else {
        return(-1);
    }
}

int
do_list(char *garbage, int com)
{
    int i, j;
    static char curr[10] = { "    " };

    if (ac == 1) {
        av[1] = "all";
        ++ac;
    }
    if (push_base()) {
        push_break();
        pop_base();
        PAGER ((void *)-1);
        pop_break();
        return (-1);
    }
    PAGER (0);
    FPAGER ("\n         ");
    for (j = 0; j < Listsize; ++j) {
        if (width[j]) {
            sprintf (Puf, "%-*.*s",
                    2 + width[j],
                    2 + width[j],
                    Find[header[j]].search);
            FPAGER (Puf);
        }
    }
    FPAGER ("\n");
    rewind_range (1);
    while ((i = get_range()) != 0) {
        i = indexof(i);
        if (Entry[i].no  &&  ((Entry[i].status & ST_DELETED) == 0)) {
            curr[0] = (Entry[i].status & ST_TAG) ? 'T' : ' ';
            curr[1] = (i == Current) ? '>' : ' ';
            curr[2] = (Entry[i].status & ST_READ)    ? 'r' : ' ';
            curr[3] = (Entry[i].status & ST_STORED)  ? 'w' : ' ';
            sprintf (Puf, "%s%-3d", curr, Entry[i].no);
            FPAGER (Puf);
            for (j = 0; j < Listsize; ++j) {
                if (width[j]) {
                    const char *s = Entry[i].fields[header[j]];
		    int l = data_line_length(s, 1) - 1;

                    if (l >= 5 && strncasecmp(Find[header[j]].search, "From:", 5) == 0) {
                        s = findSig(s, l);
			l = strlen(s);
		    }
		    if (l > width[j])
			l = width[j];

		    FPAGER("  ");
		    WPAGER(s, l);
		    if (l < width[j]) {
			l = width[j] - l;
			sprintf(Puf, "%*.*s", l, l, "");
			WPAGER(Puf, l);
		    }
                }
            }
            FPAGER ("\n");
        }
    }
    FPAGER ("\n");
    PAGER ((void *)-1);
    pop_base();
    return (1);
}

int
do_select(char *str, int mode)
{
    int ret = 1;
    int localecho = 1;
    int avi = 1;
    int scr;

    if (ac == 1)
        return (1);
    if (strcmp (av[avi], "-s") == 0) {
        localecho = 0;
        ++avi;
        --ac;
    }
    switch (ac) {
    case 2:
	{
	    int tag = 0;
	    int neg = 1;
	    const char *p = av[1];

	    if (*p == '!') {
		neg = -1;
		++p;
	    }

	    switch(*p) {
	    case 'm':
		tag = STB_READ;
		break;
	    case 'd':
		tag = STB_DELETED;
		break;
	    case 'w':
		tag = STB_STORED;
		break;
	    case 't':
		tag = STB_TAG;
		break;
	    default:
		if (localecho && !S_quiet)
		    puts ("SELECT ALL");
		break;
	    }
	    ret = m_select (Nulav + (tag != 0), mode, tag * neg);
	    scr = indexof(0);
	    if (scr > 0 && localecho && !S_quiet)
		printf ("%d  Entries selected\n", Entry[scr].no);
	}
        break;
    case 1:
        break;
    default:
        ret = m_select (av + avi, mode, 0);
        scr = indexof(0);
        if (scr > 0 && localecho && !S_quiet)
            printf ("%d  Entries selected\n", Entry[scr].no);
        break;
    }
    if (ret < 0 && localecho) {
	if (!S_quiet)
	    puts ("Null field");
        return (-1);
    }
    return (1);
}

int
do_defer(char *garbage, int com)
{
    register int i, j;

    push_break();
    j = 1;
    for (i = 0; i < Entries; ++i) {
        if (Entry[i].no)
            Entry[i].no = (Entry[i].status & ST_READ) ? 0 : j++;
    }
    pop_break();
    return (1);
}

/*
 * findSig - find text name portion of an email address
 *
 * <email> NAME
 * NAME <email>
 * email (NAME)
 * (NAME) email
 */

char *
findSig(const char *str, int l)
{
    static char buf[256];
    char *ptr = buf;
    char *t;

    if (l > sizeof(buf) - 1)
	l = sizeof(buf) - 1;

    strncpy(buf, str, l);
    buf[l] = 0;

    while (*ptr == ' ' || *ptr == '\t')
        ++ptr;
    if (*ptr == '(') {
        if ((t = strchr(ptr, ')')) != NULL)
            *t = 0;
        return(ptr + 1);
    }
    if (*ptr == '<') {
        while (*ptr && *ptr != '>')
            ++ptr;
	if (*ptr)
	    ++ptr;
	return(ptr);
    }
    while (*ptr && *ptr != '(')
        ++ptr;
    if (*ptr == '(') {
        if ((t = strchr(ptr, ')')) != NULL)
            *t = 0;
        return(ptr + 1);
    }

    if ((t = strchr(buf, '<')) != NULL) {
        *t = 0;
    }
    return(buf);
}
